module Shopapp3FormHelper
  def sf_text_field(f, tag, params = {})
    unless params.key?(:label)
      params[:label] = tag.to_s.capitalize
    end
    label_tag = f.label tag, params[:label] unless params[:label].nil?
    input_tag = f.text_field tag,
                             value: params[:value],
                             placeholder: params[:placeholder],
                             class: ["form-control", params[:class]].join(' '),
                             required: params[:required],
                             'aria-describedby' => "#{tag}_help",
                             readonly: params[:readonly],
                             onkeyup: params[:onkeyup],
                             pattern: params[:pattern],
                             title: params[:title]

    <<~HTML.html_safe
      <div class="form-group">
        #{label_flag(tag, params)}
        #{label_tag}
        #{input_tag}
        #{help_tag(tag, params)}
        #{invalid_feedback(params)}
      </div>
    HTML
  end

  def label_flag(tag, params)
    if params[:language].present?
      render partial: 'shopapp/display_flag', locals: { language: params[:language], html_class: ""}
    end
  end

  def sf_number_field(f, tag, params = {})
    unless params.key?(:label)
      params[:label] = tag.to_s.capitalize
    end
    label_tag = f.label tag, params[:label]
    input_tag = f.number_field tag,
                               value: params[:value],
                               min: 0,
                               placeholder: params[:placeholder],
                               class: 'form-control',
                               required: params[:required],
                               'aria-describedby' => "#{tag}_help"

    <<~HTML.html_safe
      <div class="form-group">
        #{label_tag}
        #{input_tag}
        #{help_tag(tag, params)}
        #{invalid_feedback(params)}
      </div>
    HTML
  end

  def sf_checkbox(f, tag, params = {})
    unless params.key?(:label)
      params[:label] = tag.to_s.capitalize
    end
    label_tag = f.label tag,
                value: params[:label],
                class: 'form-check-label'
    input_tag = f.check_box tag,
                             value: params[:value],
                             placeholder: params[:placeholder],
                             class: 'form-check-input',
                             required: params[:required],
                             'aria-describedby' => "#{tag}_help",
                             readonly: params[:readonly]

    <<~HTML.html_safe
      <div class="form-group form-check pt-2">
        #{input_tag}
        #{label_tag}
        #{help_tag(tag, params)}
        #{invalid_feedback(params)}
      </div>
    HTML
  end

  def sf_select(f, tag, params = {})
    unless params.key?(:label)
      params[:label] = tag.to_s.capitalize
    end
    label_tag = f.label tag, params[:label]
    options_list = if params.key? :enum
                     options_for_select(params[:enum].keys, params[:selected_value])
                   elsif params.key? :list
                     options_for_select(params[:list], params[:selected_value])
                   else
                     # we will impolement them as we go
                     fail NotImplementedError
                   end

    select_tag = f.select tag,
                          options_list,
                          { include_blank: (params[:blank_option_title] || "Please select...") }.merge(params[:select_params].to_h),
                          { class: "custom-select", required: params[:required] }.merge(params[:html_params].to_h)
    help_tag = if params[:help].present?
      %(<small class="form-text text-muted" id="#{tag}_help">#{params[:help]}</small>)
    end

    <<~HTML.html_safe
      <div class="form-group">
        #{label_tag}
        #{select_tag}
        #{help_tag(tag, params)}
        #{invalid_feedback(params)}
      </div>
    HTML
  end

  def sf_submit(*params_numbered, **params)
    rails ArgumentError unless params_numbered.count <= 1
    params_numbered << 'Submit' if params_numbered.count < 1
    # label = 'Submit'
    %(<button name="#{params[:name]}" type="submit" class="btn btn-primary float-right ml-2 mt-3 #{params[:class]}">#{params_numbered[0]}</button>).html_safe
  end

  def sf_cancel(return_path, label = 'Cancel', params = {})
    %(<a href="#{return_path}" class="btn btn-warning float-right ml-2 mt-3" role="button">#{label}</a>).html_safe
  end

  def sf_action(url, action_name = nil, text = nil, remote: true, method: :post, additional_classes: nil, form_class: nil, onclick: onclick)
    action_name ||= :submit
    button_to((text || action_name.to_s.humanize),
              url,
              {
                form_class: (form_class || 'float-right mr-2'),
                name: :button_action,
                method: method,
                remote: remote,
                class: ["btn", additional_classes || 'btn-secondary'].join(' '),
                data: { disable_with: "Please wait..." },
                onclick: onclick,
              }
    )
    #%button.btn.btn-danger.btn-sm Reject
  end

  def sf_model_validation(model)
    if model.errors.any?
      <<~HTML.html_safe
        <div class="alert alert-danger">
          <h5>#{pluralize(@product.errors.count, "error")} appeared in the form:</h5>
          <ul>
            #{model.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join}
          <ul>
        </div>
      HTML
    end
  end

  def sf_bool_value(value)
      symbol = value ? 'check-outline' : 'close-outline'
      mdi_tag "#{symbol} 24px"
  end

  # private helpers

  def help_tag(tag, params)
    if params[:help].present?
      %(<small class="form-text text-muted" id="#{tag}_help">#{params[:help]}</small>)
    end
  end

  def invalid_feedback(params)
    if params[:invalid_feedback].present?
      %(<div class="invalid-feedback">#{params[:invalid_feedback]}</div>)
    end

  end

  def format_js_render_remote_response(destination_selector, partial, locals = {})
    respond_to do |format|
      format.js {
        render_remote_response(destination_selector, partial, locals)
      }
    end
  end

  def render_remote_response(destination_selector, partial, locals = {})
    status = locals[:status]
    locals.delete :status
    render :render_remote_response, status: status, locals: {
      destination_selector: destination_selector,
      partial: partial.to_s,
      locals: locals
    }
  end
end
