require 'redcarpet'

module ShopappHelper
  def user_allowed?(scope)
    unless @current_user['scopes'].is_a? String
      user_scopes = @current_user['scopes']
    else
      user_scopes = JSON.parse @current_user['scopes']
    end
    user_scopes.include?('admin') || user_scopes.include?(scope.to_s)
  end

  def main_menu_link(url_path, link_text: nil, icon: nil)
    link_class_list = 'nav-link'
    link_class_list << ' active' if request.original_url.include? url_path.to_s
    link_text ||= url_path.to_s.capitalize

    full_link_text = <<~HTML.html_safe
      <i class="fa fa-#{icon}"></i>
      #{link_text}
    HTML
    <<~HTML2.html_safe
      <li class="nav-item">#{ link_to(full_link_text, url_path, class: link_class_list)}</li>
    HTML2
  end

  def secondary_menu(title, &block)
    <<~HTML2.html_safe
      <h6 class="sidebar-heading d-flex justify-content-between align-items-center px-3 mt-4 mb-1 text-muted">
        <span>#{title}</span>
      </h6>
      <ul class="nav flex-column mb-2>
        #{capture &block}
      </ul>
    HTML2
  end

  def secondary_menu_link(url_path, link_text: nil, icon: nil)
    link_text ||= url_path.to_s.capitalize
    link_class_list = 'nav-link'
    link_class_list << ' active' if request.original_url.include? url_path.to_s

    <<~HTML.html_safe
      <li class="nav-item">
        <a class="#{link_class_list}" href="#{url_path}">
          <i class="fa fa-#{icon}"></i>
          #{link_text}
        </a>
      </li>
    HTML
  end

  def search_placeholder
    controller.search_placeholder || controller.class.search_placeholder || "Search..."
  end

  def search_remote
    controller.search_remote
  end

  def search_path
    if controller.search_path == :current_path
      request.path
    elsif controller.search_path.present?
      "#{controller.search_path || '/'}"
    elsif controller.class.search_path == :current_path
      request.path
    else
      "#{controller.class.search_path || '/'}"
    end
  end

  def sidebar_item(url, text, dropdown, icon)
    <<~HTML.html_safe
      <li class="nav-item">
        <a class="nav-link #{'active' if request.path.starts_with? url}" href="#{url}">
          <i class="material-icons">#{icon}</i>
          #{text}
        </a>
      </li>
    HTML
  end

  def markdown_safe(text)
    @markdown ||= markdown = Redcarpet::Markdown.new(Redcarpet::Render::HTML, autolink: true, tables: true)
    @markdown.render(text).html_safe
  end
end
